@@ -297,5 +297,141 @@ function searchsortedlastcorrelated(v::T, x, guess::Guesser{T}) where {T <: Abst
     guess.idx_prev[] = out
     out
 end
+    
+"""
+    searchsortedfirstexp(v, x, lo=firstindex(v), hi=lastindex(v))
+
+Find the first index `i` in sorted vector `v` such that `v[i] >= x`, starting from `lo`.
+This uses an exponential search followed by binary search, which is efficient when
+the target is expected to be near `lo` (e.g., for correlated sequential lookups).
+
+Inspired by Interpolations.jl's `searchsortedfirst_exp_left`.
+"""
+Base.@propagate_inbounds function searchsortedfirstexp(
+        v::AbstractVector,
+        x,
+        lo::Integer = firstindex(v),
+        hi::Integer = lastindex(v)
+)
+    # Linear search for first few elements
+    for i in 0:4
+        ind = lo + i
+        ind > hi && return ind
+        x <= v[ind] && return ind
+    end
+    # Exponential search with doubling steps
+    n = 3
+    tn2 = 2^n
+    tn2m1 = 2^(n - 1)
+    ind = lo + tn2
+    while ind <= hi
+        x <= v[ind] &&
+            return searchsortedfirst(v, x, lo + tn2 - tn2m1, ind, Base.Order.Forward)
+        tn2 *= 2
+        tn2m1 *= 2
+        ind = lo + tn2
+    end
+    return searchsortedfirst(v, x, lo + tn2 - tn2m1, hi, Base.Order.Forward)
+end
+
+"""
+    searchsortedlastvec(v::AbstractVector, x::AbstractVector)
+
+Find the indices for multiple sorted values `x` in sorted vector `v` efficiently.
+If `x` is sorted, this leverages monotonicity to avoid redundant searching.
+Returns indices such that `v[out[i]] <= x[i]` (like `searchsortedlast`).
+
+If `x` is not sorted, falls back to element-wise `searchsortedlast`.
+
+Inspired by Interpolations.jl's `searchsortedfirst_vec`.
+"""
+function searchsortedlastvec(v::AbstractVector, x::AbstractVector)
+    issorted(x) || return searchsortedlast.(Ref(v), x)
+    out = Vector{Int}(undef, length(x))
+    lo = firstindex(v)
+    hi = lastindex(v)
+    @inbounds for i in eachindex(x)
+        xx = x[i]
+        y = searchsortedfirstexp(v, xx, lo, hi)
+        # searchsortedfirstexp returns index of first element >= x
+        # searchsortedlast returns index of last element <= x
+        # If y > hi, x is larger than all elements, return hi
+        # If v[y] == x, return y (first >= is also <=)
+        # If v[y] > x, return y - 1
+        if y > hi
+            out[i] = hi
+        elseif v[y] == xx
+            out[i] = y
+        else
+            out[i] = y - 1
+        end
+        lo = max(y, lo)
+    end
+    return out
+end
+
+"""
+    searchsortedfirstvec(v::AbstractVector, x::AbstractVector)
+
+Find the indices for multiple sorted values `x` in sorted vector `v` efficiently.
+If `x` is sorted, this leverages monotonicity to avoid redundant searching.
+Returns indices such that `v[out[i]] >= x[i]` (like `searchsortedfirst`).
+
+If `x` is not sorted, falls back to element-wise `searchsortedfirst`.
+
+Inspired by Interpolations.jl's `searchsortedfirst_vec`.
+"""
+function searchsortedfirstvec(v::AbstractVector, x::AbstractVector)
+    issorted(x) || return searchsortedfirst.(Ref(v), x)
+    out = Vector{Int}(undef, length(x))
+    lo = firstindex(v)
+    hi = lastindex(v)
+    @inbounds for i in eachindex(x)
+        xx = x[i]
+        y = searchsortedfirstexp(v, xx, lo, hi)
+        out[i] = y
+        lo = min(y, hi)
+    end
+    return out
+end
+
+using PrecompileTools: @compile_workload, @setup_workload
+    
+@setup_workload begin
+    # Minimal setup for precompilation workload
+    vec_int64 = Int64[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]
+    linear_vec = collect(1.0:0.5:10.0)
+
+    @compile_workload begin
+        # Precompile the most commonly used functions with typical types
+
+        # findfirstequal: fast SIMD-based search in Int64 vectors
+        findfirstequal(Int64(5), vec_int64)
+        findfirstequal(Int64(100), vec_int64)  # not found case
+
+        # findfirstsortedequal: binary search in sorted Int64 vectors
+        findfirstsortedequal(Int64(8), vec_int64)
+        findfirstsortedequal(Int64(100), vec_int64)  # not found case
+
+        # bracketstrictlymontonic: bracketing for sorted vectors
+        bracketstrictlymontonic(vec_int64, Int64(8), Int64(1), Base.Order.Forward)
+
+        # looks_linear: check if vector is evenly spaced
+        looks_linear(linear_vec)
+
+        # Guesser: wrapper for efficient repeated searches
+        guesser = Guesser(linear_vec)
+        guesser(5.0)
+
+        # searchsortedfirstcorrelated and searchsortedlastcorrelated
+        searchsortedfirstcorrelated(vec_int64, Int64(8), Int64(1))
+        searchsortedlastcorrelated(vec_int64, Int64(8), Int64(1))
+
+        # Also precompile with Guesser
+        guesser_int = Guesser(vec_int64)
+        searchsortedfirstcorrelated(vec_int64, Int64(8), guesser_int)
+        searchsortedlastcorrelated(vec_int64, Int64(8), guesser_int)
+    end
+end
 
 end # module FindFirstFunctions
